;###############################################################################
;###################          Gaussian Smoothing            ####################

(define vigra_gaussiansmoothing_c
  (get-ffi-obj 'vigra_gaussiansmoothing_c dylib-path
               (_fun (img_vector1 img_vector2  width height sigma) :: [img_vector1 : _cvector]
                     [img_vector2 : _cvector]
                     [width : _int]
                     [height : _int]
                     [sigma : _float]
                     -> (res :  _int))))

(define (gsmooth image sigma)
  (let* ((width  (image-width  image))
	 (height (image-height image))
	 (img2   (make-image width height 0.0))
	 (foo   (vigra_gaussiansmoothing_c (image-data image) (image-data img2) width height sigma)))
    (case foo
      ((0) img2)
      ((1) (error "Error in vigraplt.filters.gsmooth: Gaussian smoothing failed!")))))


;###############################################################################
;###################      Gaussian Gradient (Magnitude)     ####################

(define vigra_gaussiangradientmagnitude_c
  (get-ffi-obj 'vigra_gaussiangradientmagnitude_c dylib-path
               (_fun (img_vector1 img_vector2  width height sigma) :: [img_vector1 : _cvector]
                     [img_vector2 : _cvector]
                     [width : _int]
                     [height : _int]
                     [sigma : _float]
                     -> (res :  _int))))

(define (ggradient image sigma)
  (let* ((width  (image-width  image))
	 (height (image-height image))
	 (img2   (make-image width height 0.0))
	 (foo   (vigra_gaussiangradientmagnitude_c (image-data image) (image-data img2) width height sigma)))
    (case foo
      ((0) img2)
      ((1) (error "Error in vigraplt.filters.ggradient: Gaussian gradient magnitude failed!")))))


;###############################################################################
;###################        Laplacian Of Gaussian           ####################

(define vigra_laplacianofgaussian_c
  (get-ffi-obj 'vigra_laplacianofgaussian_c dylib-path
               (_fun (img_vector1 img_vector2  width height scale) :: [img_vector1 : _cvector]
                     [img_vector2 : _cvector]
                     [width : _int]
                     [height : _int]
                     [scale : _float]
                     -> (res :  _int))))

(define (laplacianofgaussian image scale)
  (let* ((width  (image-width  image))
	 (height (image-height image))
	 (img2   (make-image width height 0.0))
	 (foo   (vigra_laplacianofgaussian_c (image-data image) (image-data img2) width height scale)))
    (case foo
      ((0) img2)
      ((1) (error "Error in vigraplt.filters.laplacianofgaussian: Laplacian of Gaussian failed!")))))



;###############################################################################
;###################          Gaussian Sharpening           ####################

(define vigra_gaussiansharpening_c
  (get-ffi-obj 'vigra_gaussiansharpening_c dylib-path
               (_fun (img_vector1 img_vector2  width height sharpening_factor scale) :: [img_vector1 : _cvector]
                     [img_vector2 : _cvector]
                     [width : _int]
                     [height : _int]
                     [sharpening_factor : _float]
                     [scale : _float]
                     -> (res :  _int))))

(define (gsharpening image sharpening_factor scale)
  (let* ((width  (image-width  image))
	 (height (image-height image))
	 (img2   (make-image width height 0.0))
	 (foo   (vigra_gaussiansharpening_c (image-data image) (image-data img2) width height  sharpening_factor scale)))
    (case foo
      ((0) img2)
      ((1) (error "Error in vigraplt.filters.gsharpening: Gaussian sharpening failed!")))))


;###############################################################################
;###################            Simple Sharpening           ####################

(define vigra_simplesharpening_c
  (get-ffi-obj 'vigra_simplesharpening_c dylib-path
               (_fun (img_vector1 img_vector2  width height sharpening_factor) :: [img_vector1 : _cvector]
                     [img_vector2 : _cvector]
                     [width : _int]
                     [height : _int]
                     [sharpening_factor : _float]
                     -> (res :  _int))))

(define (sharpening image sharpening_factor)
  (let* ((width  (image-width  image))
	 (height (image-height image))
	 (img2   (make-image width height 0.0))
	 (foo   (vigra_simplesharpening_c (image-data image) (image-data img2) width height sharpening_factor)))
    (case foo
      ((0) img2)
      ((1) (error "Error in vigraplt.filters.sharpening: Simple sharpening failed!")))))



;###############################################################################
;###################          Nonlinear Diffusion           ####################

(define vigra_nonlineardiffusion_c
  (get-ffi-obj 'vigra_nonlineardiffusion_c dylib-path
               (_fun (img_vector1 img_vector2  width height edge_threshold scale) :: [img_vector1 : _cvector]
                     [img_vector2 : _cvector]
                     [width : _int]
                     [height : _int]
                     [edge_threshold : _float]
                     [scale : _float]
                     -> (res :  _int))))

(define (nonlineardiffusion image edge_threshold scale)
  (let* ((width  (image-width  image))
	 (height (image-height image))
	 (img2   (make-image width height 0.0))
	 (foo   (vigra_nonlineardiffusion_c (image-data image) (image-data img2) width height edge_threshold scale)))
    (case foo
      ((0) img2)
      ((1) (error "Error in vigracl.filters.nonlineardiffusion: Non linear Diffusion failed!!")))))

;###############################################################################
;###################          Distance Transform            ####################

(define vigra_distancetransform_c
  (get-ffi-obj 'vigra_distancetransform_c dylib-path
               (_fun (img_vector1 img_vector2  width height background_label norm) :: [img_vector1 : _cvector]
                     [img_vector2 : _cvector]
                     [width : _int]
                     [height : _int]
                     [background_label : _float]
                     [norm : _int]
                     -> (res :  _int))))

(define (distancetransform image background_label norm)
  (let* ((width  (image-width  image))
	 (height (image-height image))
	 (img2   (make-image width height 0.0))
	 (foo   (vigra_distancetransform_c (image-data image) (image-data img2) width height background_label norm)))
    (case foo
      ((0) img2)
      ((1) (error "Error in vigraplt.filters.distancetransform: Distance transformation failed!!"))
      ((2) (error "Error in vigraplt.filters.distancetransform: Norm must be in {0,1,2} !!")))))