;###############################################################################
;###################         Label image                    ####################

(define vigra_labelimage_c
  (get-ffi-obj 'vigra_labelimage_c dylib-path
               (_fun (img_vector1 img_vector2  width height) :: [img_vector1 : _cvector]
                     [img_vector2 : _cvector]
                     [width : _int]
                     [height : _int]
                     -> (res :  _int))))

(define (labelimage image)
  (let* ((width  (image-width  image))
	 (height (image-height image))
	 (img2 (make-image width height 0.0))
	 (foo   (vigra_labelimage_c (image-data image) (image-data img2) width height)))
    (if (= foo -1)
        (error "Error in vigraplt.segmentation.labelimage: Labeling of image failed!")
        img2)))
	  
	  
;###############################################################################
;###################      Watershed Transform (Union-Find)  ####################

(define vigra_watersheds_c
  (get-ffi-obj 'vigra_watersheds_c dylib-path
               (_fun (img_vector1 img_vector2  width height) :: [img_vector1 : _cvector]
                     [img_vector2 : _cvector]
                     [width : _int]
                     [height : _int]
                     -> (res :  _int))))

(define (watersheds image)
  (let* ((width  (image-width  image))
	 (height (image-height image))
	 (img2 (make-image width height 0.0))
	 (foo   (vigra_watersheds_c (image-data image) (image-data img2) width height)))
    (if (= foo -1)
        (error	"Error in vigracl.segmentation.watersheds: Watershed Transform of image failed!")
        img2)))
	  
	  
;###############################################################################
;###################      Canny Edge-Detection              ####################

(define vigra_cannyedgeimage_c
  (get-ffi-obj 'vigra_cannyedgeimage_c dylib-path
               (_fun (img_vector1 img_vector2  width height  scale gradient_threshold mark) :: [img_vector1 : _cvector]
                     [img_vector2 : _cvector]
                     [width : _int]
                     [height : _int]
                     [scale : _float]
                     [gradient_threshold  : _float]
                     [mark : _float]
                     -> (res :  _int))))

(define (cannyedgeimage image scale gradient_threshold mark)
  (let* ((width  (image-width  image))
	 (height (image-height image))
	 (img2 (make-image width height 0.0))
	 (foo   (vigra_cannyedgeimage_c (image-data image) (image-data img2) width height scale gradient_threshold mark)))
    (case foo
      ((0) img2)
      ((1) (error "Error in vigraplt.segmentation:cannyedgeimage: Canny Edge Detection of image failed!")))))
	  
	  
;###############################################################################
;################    Difference of Exponential Edge-Detection  #################

(define vigra_differenceofexponentialedgeimage_c
  (get-ffi-obj 'vigra_differenceofexponentialedgeimage_c dylib-path
               (_fun (img_vector1 img_vector2  width height  scale gradient_threshold mark) :: [img_vector1 : _cvector]
                     [img_vector2 : _cvector]
                     [width : _int]
                     [height : _int]
                     [scale : _float]
                     [gradient_threshold  : _float]
                     [mark : _float]
                     -> (res :  _int))))

(define (differenceofexponentialedgeimage image scale gradient_threshold mark)
  (let* ((width  (image-width  image))
	 (height (image-height image))
	 (img2 (make-image width height 0.0))
	 (foo   (vigra_differenceofexponentialedgeimage_c (image-data image) (image-data img2) width height scale gradient_threshold mark)))
    (case foo
      ((0) img2)
      ((1) (error "Error in vigraplt.segmentation:differenceofexponentialedgeimage: Difference of Exponential Edge Detection of image failed!")))))


;###############################################################################
;###################     RegionImage -> CrackEdgeImage      ####################

(define vigra_regionimagetocrackedgeimage_c
  (get-ffi-obj 'vigra_regionimagetocrackedgeimage_c dylib-path
               (_fun (img_vector1 img_vector2  width height mark) :: [img_vector1 : _cvector]
                     [img_vector2 : _cvector]
                     [width : _int]
                     [height : _int]
                     [mark : _float]
                     -> (res :  _int))))

(define (regionimagetocrackedgeimage image mark)
  (let* ((width  (image-width  image))
	 (height (image-height image))
	 (img2 (make-image (- (* 2 width) 1) (- (* 2 height) 1) 0.0))
	 (foo   (vigra_regionimagetocrackedgeimage_c (image-data image) (image-data img2) width height mark)))
    (case foo
      ((0) img2)
      ((1) (error "Error in vigraplt.segmentation:regionimagetocrackedgeimage: Creation of CrackEdgeImage failed!")))))