                             ;Dependencies:
(require       mred          ;Needed for bitmaps-im/export
               mzlib/class)  ;Non-CLOS (MzScheme) Classes  + DCs
         
  
;;writes an array of color values back to a given bitmap device context
;;the width and height of the image represented in the list are 
;;assumed to be in the Dimensions of the given bitmap_dc as the list
;;alone has no furter information about the shape of the underlying
;;images color values.
;;   Example Array:  ( 1 2 3 )                   Bitmap : ( 1 2 3 )     
;;                   ( 4 5 6 )             ===>           ( 4 5 6 )  
;;                   ( 7 8 9 )                            ( 7 8 9 )
;; @param pixelvector   the 1-dimensional vector of pixel colors
;; @param bitmap_dc     the bitmap's device context
;; @return            #t if successfully finished, error else
(define (image->bitmap_dc image bitmap_dc)
  (let* ((height (image-height image))
         (width  (image-width image))
         (fill_bdc (lambda (x y)
                     (let* ((element      (inexact->exact (max (min (round(image-ref image x y)) 255) 0)))
                            (pixel_color   (make-object color% element
                                                               element
                                                               element)))
                       (send bitmap_dc set-pixel x y pixel_color)))))
    (image-for-each-index fill_bdc image)))

;;#############################################################################
;;####         GUI for showing images. Clicking inside the image            ###
;;####         shows the current pixel values!                              ###
;;#############################################################################

;;Creates a frame, places a canvas inside it  and draws the given pixelarray
;;to the canvas each time the redraw-event is raised.
;;The pixelarray is transformed to a bitmap resp. bitmap_dc to enable
;;painting. Currently, RGB- and Gray-Images are supported
(define (show-image image . windowtitle)
  (let* ((width  (image-width image))
         (height (image-height image))
         ;Create a temp. bitmap
         (bitmap    (make-object bitmap% width height #f))
         ;Define a device context for pixel extraction and link it to the basicimage
         (bitmap_dc (new bitmap-dc%))
         (foo  (send bitmap_dc set-bitmap bitmap))
         (bar  (image->bitmap_dc image bitmap_dc)))
    (let-values ([(screen_w screen_h) (get-display-size)])
    
      (begin
        ;; Make a  frame
        (define frame
          (new frame% [label (if (pair? windowtitle) (car  windowtitle) "VigraPLT Image Viewer")]
               [width (min screen_w (+ width 20))] [height (min screen_h (+ height 70))]))
        ;; Make the drawing area
        (define canvas (new
                 ;; --- create a class on the fly that overrides the  callback for
                 ;; --- mouse events in a canvas; you could name it,  if needed
                 (class canvas%
                   ;; Event -> Void
                   ;; act on each mouse event
                   (define/override (on-event e)
                     (define kind (send e moving?))
                     (define x (send e get-x))
                     (define y (send e get-y))
                     (define c (new color%))
                     (when kind
                       (if (send bitmap_dc get-pixel x y c)
                           ;;for gray-images
                           (send frame set-status-text (string-append "Value at pixel ("
                                                                      (number->string x)
                                                                      ","
                                                                      (number->string y)
                                                                      ") is : "
                                                                      (number->string (image-ref image x y))))
                           ;;errorcase
                           (send frame set-status-text (string-append "Value at pixel ("
                                                                      (number->string x)
                                                                      ","
                                                                      (number->string y)
                                                                      ") is not defined!")))))
                   (super-new))
                 ;;--- provide a frame and a paint method for  displaying plt.gif
                 [parent frame]
                 [style '(vscroll hscroll)]
                 [paint-callback (lambda (e dc) (send dc draw-bitmap bitmap 0 0))]));; Get the canvas's device context
        (send canvas init-auto-scrollbars width height 0.0 0.0)
        ;;enable a status line
        (send frame create-status-line)
        ;; Show the frame
        (send frame show #t)))))